﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

using Quartz;
using Quartz.Core;
using Quartz.Impl;
using Quartz.Listener;

namespace TopshelfStudy.NetCoreDemo
{
    /// <summary>
    /// Quarzt触发监听器
    /// </summary>
    public class CustomTriggerListener : ITriggerListener
    {
        public readonly NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();

        public string Name { get; } = nameof(CustomTriggerListener);

        public CustomTriggerListener()
        {
        }

        /// <summary>
        /// 在Quartz.ITrigger触发时由Quartz.IScheduler调用
        /// 它的关联Quartz.IJobDetail已执行
        /// 它的Quartz.Spi.IOperableTrigger.trigged（Quartz.ICalendar）方法已调用
        /// </summary>
        public Task TriggerComplete(ITrigger trigger, IJobExecutionContext context, SchedulerInstruction triggerInstructionCode, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info("触发器执行后");
            return Task.CompletedTask;
        }

        /// <summary>
        /// 触发器启动时执行
        /// 当Quartz.ITrigger启动时由Quartz.IScheduler调用，它的关联Quartz.IJobDetail即将执行。
        /// 它在VetoJobExecution()方法之前调用
        /// </summary>
        public Task TriggerFired(ITrigger trigger, IJobExecutionContext context, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info("触发器执行前");
            return Task.CompletedTask;
        }

        /// <summary>
        /// 错过触发时调用
        /// 当Quartz.ITrigger错过触发时(线程池不够且等待超时)调用时，由Quartz.IScheduler调用。
        /// 应该考虑这个方法花费了多少时间，因为将影响所有错过触发的触发器。
        /// 如果你有很多的触发器错过触发，这可能是这个方法做了很多事情。
        /// </summary>
        public Task TriggerMisfired(ITrigger trigger, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info("错过触发时间，本次任务丢失.");
            return Task.CompletedTask;
        }

        /// <summary>
        /// 在Quartz.ITrigger触发时由Quartz.IScheduler调用
        /// 它的关联Quartz.IJobDetail已执行，它的Quartz.Spi.IOperableTrigger.trigged（Quartz.ICalendar）方法已调用
        /// </summary>
        /// <returns>
        /// true表示否决Job继续执行,false同意继续执行
        /// </returns>
        public Task<bool> VetoJobExecution(ITrigger trigger, IJobExecutionContext context, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info("触发器审核通过,继续执行.");
            return Task.FromResult(false); 
        }
    }
}
